﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.ServiceModel;
using System.Text;
using System.Threading.Tasks;

namespace FC12SamlExample
{
    class Program
    {
        private const string SAMLServiceUrl = "http://localhost/Flexicapture12/Server/SAML";
        private const string APIServiceUrl = "http://localhost/FlexiCapture12/Server/FCAuth/API/v1/Soap";

        public static async Task Main( string[] args )
        {
            // Получаем первый идентификационный билет от конечной точки FC 12 SAML 
            string authTicket = await AuthenticateWithSAMLAsync( SampleSamlData.SamlRequest );

            var client = new FC12Service.FlexiCaptureWebServiceSoapClient(
                new BasicHttpBinding(), new EndpointAddress(APIServiceUrl)
            );
            
            // настроенное поведение добавляет идентификационный заголовок к каждому вызову сервиса
            client.Endpoint.EndpointBehaviors.Add(new HttpAuthHeaderEndpointBehavior(authTicket));

            // через методы API
            int sesisonId = client.OpenSession(1/*Scanning operator*/, 1/*Scanning station*/ );

            client.CloseSession(sesisonId);
        }

        static async Task<string> AuthenticateWithSAMLAsync( string samlData )
        {
            var fields = new Dictionary<string, string>
            {
                { "SAMLResponse", Convert.ToBase64String( Encoding.UTF8.GetBytes( samlData ) ) }
            };
            var content = new FormUrlEncodedContent( fields );

            using ( var client = new HttpClient() )
            {
                HttpResponseMessage response = await client.PostAsync( SAMLServiceUrl, content );

                if ( response.StatusCode == HttpStatusCode.OK )
                {
                    return response.Headers.GetValues("AuthTicket").FirstOrDefault() 
                        ?? throw new Exception("No AuthTicket found");
                }
                else
                {
                    throw new Exception( await response.Content.ReadAsStringAsync() );
                }
            }
        }
    }
}
